==> bloc_stream-4.2.1.tar.gz: lib/bloc_stream.dart <==
library bloc_stream;

import 'dart:async';

import 'package:meta/meta.dart';
import 'package:rxdart/rxdart.dart';

typedef BlocStreamAction<State, B extends BlocStream<State>> = Stream<State>
    Function(
  State currentValue,
  B bloc,
);

abstract class BlocStream<State> extends Stream<State> {
  BlocStream(State initialState)
      : _state$ = BehaviorSubject.seeded(initialState) {
    _bindActionsToStates();
  }

  final _actionController = StreamController<dynamic>();

  final BehaviorSubject<State> _state$;
  State get value => _state$.value;

  StreamSubscription<State> _transformSubscription;

  @override
  bool get isBroadcast => _state$.isBroadcast;

  @override
  StreamSubscription<State> listen(
    void Function(State) onData, {
    Function onError,
    void Function() onDone,
    bool cancelOnError,
  }) {
    return _state$.listen(
      onData,
      onError: onError,
      onDone: onDone,
      cancelOnError: cancelOnError,
    );
  }

  /// Add actions to be processed
  void add<B extends BlocStream<State>>(BlocStreamAction<State, B> action) {
    if (_state$.isClosed) return;
    _actionController.add(action as dynamic);
  }

  void willUpdateValue(State current, State next) {}

  Stream<State> _transformAction(dynamic action) => action(_state$.value, this);

  /// Transforms a stream of actions to a stream of state.
  /// You can override this to throttle actions for example.
  Stream<State> transformActions(
    Stream<dynamic> actions$,
    Stream<State> Function(BlocStreamAction<State, BlocStream<State>>)
        transform,
  ) =>
      actions$.asyncExpand(transform);

  void _bindActionsToStates() {
    _transformSubscription = transformActions(
      _actionController.stream,
      _transformAction,
    ).listen((newState) {
      if (newState == _state$.value) return;
      willUpdateValue(_state$.value, newState);
      _state$.add(newState);
    }, onError: (error) {
      _state$.addError(error);
    });
  }

  /// Helper method to reduce boilerplate when composing BLoC's together.
  final _closers = <void Function()>[];
  void cancelOnClose(StreamSubscription subscription,
      [void Function() onClose]) {
    _closers.add(() {
      subscription.cancel();
      if (onClose != null) onClose();
    });
  }

  @mustCallSuper
  Future<void> close() {
    _closers.forEach((fn) => fn());
    _transformSubscription.cancel();
    _actionController.close();
    return _state$.close();
  }
}

==> bloc_stream-4.2.3.tar.gz: lib/bloc_stream.dart <==
library bloc_stream;

import 'dart:async';

import 'package:meta/meta.dart';
import 'package:rxdart/rxdart.dart';

typedef BlocStreamAction<State, B extends BlocStream<State>> = Stream<State>
    Function(
  State currentValue,
  B bloc,
);

abstract class BlocStream<State> extends Stream<State> {
  BlocStream(State initialState) {
    _state$ = BehaviorSubject.seeded(initialValueOverride ?? initialState);
    _bindActionsToStates();
  }

  State get initialValueOverride => null;

  final _actionController = StreamController<dynamic>();

  BehaviorSubject<State> _state$;
  State get value => _state$.value;

  StreamSubscription<State> _transformSubscription;

  @override
  bool get isBroadcast => _state$.isBroadcast;

  @override
  StreamSubscription<State> listen(
    void Function(State) onData, {
    Function onError,
    void Function() onDone,
    bool cancelOnError,
  }) {
    return _state$.listen(
      onData,
      onError: onError,
      onDone: onDone,
      cancelOnError: cancelOnError,
    );
  }

  /// Add actions to be processed
  void add<B extends BlocStream<State>>(BlocStreamAction<State, B> action) {
    if (_state$.isClosed) return;
    _actionController.add(action as dynamic);
  }

  void willUpdateValue(State current, State next) {}

  Stream<State> _transformAction(dynamic action) => action(value, this);

  /// Transforms a stream of actions to a stream of state.
  /// You can override this to throttle actions for example.
  Stream<State> transformActions(
    Stream<dynamic> actions$,
    Stream<State> Function(BlocStreamAction<State, BlocStream<State>>)
        transform,
  ) =>
      actions$.asyncExpand(transform);

  void _bindActionsToStates() {
    _transformSubscription = transformActions(
      _actionController.stream,
      _transformAction,
    ).listen((newState) {
      if (newState == value) return;
      willUpdateValue(value, newState);
      _state$.add(newState);
    }, onError: (error) {
      _state$.addError(error);
    });
  }

  /// Helper method to reduce boilerplate when composing BLoC's together.
  final _closers = <void Function()>[];
  void cancelOnClose(StreamSubscription subscription,
      [void Function() onClose]) {
    _closers.add(() {
      subscription.cancel();
      if (onClose != null) onClose();
    });
  }

  @mustCallSuper
  Future<void> close() {
    _closers.forEach((fn) => fn());
    _transformSubscription.cancel();
    _actionController.close();
    return _state$.close();
  }
}
